@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.165 2010/04/29 11:00:41 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.166 2010/08/17 05:53:34 lav Exp $ */
 
 #include <config.h>
 
@@ -563,7 +563,7 @@ void FileAccess::Chmod(const char *file,int m)
 
 void FileAccess::SetError(int ec,const char *e)
 {
-   if(ec==SEE_ERRNO)
+   if(ec==SEE_ERRNO && !saved_errno)
       saved_errno=errno;
    if(ec==NO_FILE && file && file[0] && !strstr(e,file))
       error.vset(e," (",file.get(),")",NULL);
@@ -574,6 +574,7 @@ void FileAccess::SetError(int ec,const char *e)
 
 void FileAccess::ClearError()
 {
+   saved_errno=0;
    error_code=OK;
    error.set(0);
 }
