@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.231 2008/03/05 06:39:43 lav Exp $ */
+/* $Id: Http.cc,v 1.237 2010/08/17 05:53:34 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -26,12 +26,12 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <fcntl.h>
-#include <assert.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <time.h>
 #include <fnmatch.h>
 #include <locale.h>
+#include <assert.h>
 #include "Http.h"
 #include "ResMgr.h"
 #include "log.h"
@@ -236,10 +236,13 @@ void Http::Send(const char *format,...)
 
 void Http::SendMethod(const char *method,const char *efile)
 {
-   const char *ehost=xstring::join(":",2,
-      url::encode(hostname,URL_HOST_UNSAFE),
-      url::encode(portname,URL_PORT_UNSAFE)
-   );
+   xstring& stripped_hostname=xstring::get_tmp(hostname);
+   stripped_hostname.truncate_at('%');
+   xstring& ehost=url::encode(stripped_hostname,URL_HOST_UNSAFE);
+   if(portname) {
+      ehost.append(':');
+      ehost.append(url::encode(portname,URL_PORT_UNSAFE));
+   }
    if(!use_head && !strcmp(method,"HEAD"))
       method="GET";
    last_method=method;
@@ -273,7 +276,7 @@ void Http::SendMethod(const char *method,const char *efile)
       efile="/";
 
    Send("%s %s HTTP/1.1\r\n",method,efile);
-   Send("Host: %s\r\n",ehost);
+   Send("Host: %s\r\n",ehost.get());
    if(user_agent && user_agent[0])
       Send("User-Agent: %s\r\n",user_agent);
    if(!hftp)
@@ -309,7 +312,7 @@ void Http::SendMethod(const char *method,const char *efile)
 
       xstring cookie;
       MakeCookie(cookie,hostname,efile+(proxy?url::path_index(efile):0));
-      if(cookie && cookie[0])
+      if(cookie.length()>0)
 	 Send("Cookie: %s\r\n",cookie.get());
    }
 }
@@ -587,7 +590,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	    SendMethod("PROPFIND",efile);
 	    Send("Depth: 0\r\n"); // no directory listing required
 	    Send("Content-Type: text/xml\r\n");
-	    Send("Content-Length: %d\r\n",strlen(allprop));
+	    Send("Content-Length: %d\r\n",(int)strlen(allprop));
 	 }
 	 else
 	    SendMethod("HEAD",efile);
@@ -610,7 +613,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	 SendMethod("PROPFIND",efile);
 	 Send("Depth: 1\r\n"); // directory listing required
 	 Send("Content-Type: text/xml\r\n");
-	 Send("Content-Length: %d\r\n",strlen(allprop));
+	 Send("Content-Length: %d\r\n",(int)strlen(allprop));
 	 pos=0;
       }
       break;
@@ -1022,13 +1025,14 @@ int Http::Do()
       sock=SocketCreateTCP(peer[peer_curr].sa.sa_family);
       if(sock==-1)
       {
+	 saved_errno=errno;
 	 if(peer_curr+1<peer.count())
 	 {
 	    peer_curr++;
 	    retries--;
 	    return MOVED;
 	 }
-	 if(NonFatalError(errno))
+	 if(NonFatalError(saved_errno))
 	    return m;
 	 char str[256];
 	 sprintf(str,_("cannot create socket of address family %d"),
@@ -1036,20 +1040,16 @@ int Http::Do()
 	 SetError(SEE_ERRNO,str);
 	 return MOVED;
       }
-      KeepAlive(sock);
-      SetSocketBuffer(sock);
-      SetSocketMaxseg(sock);
-      NonBlock(sock);
-      CloseOnExec(sock);
 
       SayConnectingTo();
       res=SocketConnect(sock,&peer[peer_curr]);
       if(res==-1 && errno!=EINPROGRESS)
       {
+	 saved_errno=errno;
 	 NextPeer();
-	 LogError(0,"connect: %s\n",strerror(errno));
+	 LogError(0,"connect: %s\n",strerror(saved_errno));
 	 Disconnect();
-	 if(NotSerious(errno))
+	 if(NotSerious(saved_errno))
 	    return MOVED;
 	 goto system_error;
       }
@@ -1144,10 +1144,7 @@ int Http::Do()
       state=RECEIVING_HEADER;
       m=MOVED;
       if(mode==STORE)
-      {
-	 assert(rate_limit==0);
 	 rate_limit=new RateLimit(hostname);
-      }
 
    case RECEIVING_HEADER:
       if(send_buf->Error() || recv_buf->Error())
@@ -1529,7 +1526,8 @@ int Http::Do()
    return m;
 
 system_error:
-   if(NonFatalError(errno))
+   assert(saved_errno!=0);
+   if(NonFatalError(saved_errno))
       return m;
    SetError(SEE_ERRNO,0);
    Disconnect();
@@ -1988,45 +1986,32 @@ void Http::CookieMerge(xstring &all,const char *cookie_c)
 	 c_value=c_name, c_name=0;
       int c_name_len=xstrlen(c_name);
 
-      char *scan=all.get_non_const();
-      for(;;)
+      for(unsigned i=all.skip_all(0,' '); i<all.length(); i=all.skip_all(i+1,' '))
       {
-	 while(*scan==' ') scan++;
-	 if(*scan==0)
-	    break;
-
-	 char *semicolon=strchr(scan,';');
-	 char *eq=strchr(scan,'=');
+	 const char *scan=all+i;
+	 const char *semicolon=strchr(scan,';');
+	 const char *eq=strchr(scan,'=');
 	 if(semicolon && eq>semicolon)
 	    eq=0;
 	 if((eq==0 && c_name==0)
 	 || (eq-scan==c_name_len && !strncmp(scan,c_name,c_name_len)))
 	 {
 	    // remove old cookie.
-	    const char *m=semicolon?semicolon+1:"";
-	    while(*m==' ') m++;
-	    if(*m==0)
-	    {
-	       while(scan>all && scan[-1]==' ')
-		  scan--;
-	       if(scan>all && scan[-1]==';')
-		  scan--;
-	       *scan=0;
-	    }
+	    if(!semicolon)
+	       all.truncate(i);
 	    else
-	       memmove(scan,m,strlen(m)+1);
+	       all.set_substr(i,all.skip_all(semicolon+1-all,' ')-i,"",0);
 	    break;
 	 }
 	 if(!semicolon)
 	    break;
-	 scan=semicolon+1;
+	 i=semicolon+1-all;
       }
 
       // append cookie.
-      int c_len=strlen(all);
-      while(c_len>0 && all[c_len-1]==' ')
-	 c_len--; // trim
-      all.truncate(c_len);
+      all.rtrim(' ');
+      all.rtrim(';');
+      int c_len=all.length();
       if(c_len>0 && all[c_len-1]!=';')
 	 all.append("; ");
       if(c_name)
@@ -2194,7 +2179,7 @@ void Http::LogErrorText()
    if(!recv_buf)
       return;
    recv_buf->Roll();
-   size_t size=recv_buf->Size();
+   int size=recv_buf->Size();
    if(size==0)
       return;
    char *buf=string_alloca(size+1);
