@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,10 +21,10 @@
 /* All indexes in this function start at 0; -1 is used contextually to
  * indicate the last job or moving to the end of the list. */
 #include <config.h>
-#include <errno.h>
 #include <unistd.h>
 #include <assert.h>
 #include <fnmatch.h>
+#include <stddef.h>
 
 #include "QueueFeeder.h"
 #include "plural.h"
