@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.78 2010/04/29 11:00:42 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.79 2010/09/01 09:33:03 lav Exp $ */
 
 #include <config.h>
 
@@ -538,7 +538,8 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
       return true;
    if(!(closure && cl_data))
       return false;
-   return 0==fnmatch(closure,cl_data,FNM_PATHNAME);
+   return 0==fnmatch(closure,cl_data,FNM_PATHNAME)
+      || (closure[0]=='*' && closure[1]=='.' && !strcmp(closure+2,cl_data));
 }
 
 const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
