@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2001-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -212,3 +212,10 @@ const char *TimeInterval::toString(unsigned flags)
    }
    return buf;
 }
+
+const TimeDiff& TimeDiff::valueOf(double v)
+{
+   static TimeDiff diff;
+   diff.Set(v);
+   return diff;
+}
